import json

from django.conf import settings

from django.urls import reverse
from rest_framework import status

from tropipay.views import make_payment
from rest_framework.test import APIClient, APITestCase
from django.core.management import call_command
import environ
from django.contrib.auth import get_user_model
from model_bakery import baker, random_gen


class BaseTestCase(APITestCase):
    @staticmethod
    def get_base_url() -> str:
        return settings.BASE_URL


    @staticmethod
    def get_root_dir() -> str:
        import os
        env = environ.Env()
        return "./"

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

    def setUp(self):

        super().setUp()
        self.client = APIClient()

        if self.fixtures:
            call_command(
                "loaddata_custom",
                *self.fixtures,
                **{"verbosity": 0, "schema": "test"},
            )

        self.user = get_user_model().objects.create_user(
            username="test",
            email=baker.random_gen.gen_email().lower(),
            password="pass",
            is_staff=True,
            is_superuser=True,
        )


class TestPayment(BaseTestCase):
    def test_make_payment(self):
        url = f"{self.get_base_url()}/tropipay/pay/"
        data = {
            "uuid":"test_uuid",
            "title":"Test Title",
            "name":"Test Name",
            "last_name":"Test Last Name",
            "email":"blinit10@gmail.com",
            "phone_number":"+535235131",
            "description":"Test Description",
            "location":"Test Location",
            "total": 30.00,

        }
        response = self.client.post(url, data, format="json")
        self.assertIsNotNone(response.json().get("payment_link"))



from django.urls import path

from tropipay.views import make_payment, success, verify

urlpatterns = [
    path('pay/', make_payment, name='make_payment'),
    path('verify/', verify, name='verify'),
    path('success/', success, name='success'),
]


import json
from django.conf import settings
from hashlib import sha1, sha256
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

from tropipay.common import config_exists, get_config, ORDER_KEYS, get_payment_payload, perform_payment


def create_order(purchase_data, **kwargs):
    for key in ORDER_KEYS:
        if purchase_data.get(key) is None:
            return None
    return {key: purchase_data.get(key) for key in ORDER_KEYS}


@api_view(['POST'])
@csrf_exempt
def make_payment(request):
    if config_exists():
        tropipay_config = get_config()
        purchase_data = json.loads(request.body.decode('utf-8'))
        order = create_order(purchase_data, **{})
        if order is not None:
            return perform_payment(tropipay_config, get_payment_payload(tropipay_config, order))
    return Response(status=status.HTTP_400_BAD_REQUEST)


@method_decorator(csrf_exempt)
def success(request):
    uuid: str = request.GET.get('reference').replace('-', '')
    if uuid is None:
        return Response(status=status.HTTP_400_BAD_REQUEST)
    return redirect(
        f'https://api.whatsapp.com/send/?phone={settings.WHATSAPP_REDIRECT_PHONE}&text="Hello!, Just made the order with ticket {uuid}"&app_absent=1')


@method_decorator(csrf_exempt)
def verify(request):
    if request.method == 'POST':
        tropipay_config: dict = get_config()
        payload = json.loads(request.body)
        bankOrderCode = payload['data']['bankOrderCode']
        originalCurrencyAmount = payload['data']['originalCurrencyAmount']
        signature = payload['data']['signature']
        payload_status = payload['status']
        reference = payload['data']['reference']
        constructed_signature = sha256(
            f"{bankOrderCode}{tropipay_config.get("TROPIPAY_CLIENT_EMAIL")}{sha1(tropipay_config.get("TROPIPAY_CLIENT_PASSWORD").encode('utf-8')).hexdigest()}{originalCurrencyAmount}".encode(
                'utf-8')).hexdigest()
        if constructed_signature == signature:
            return Response(status=status.HTTP_200_OK, data={"reference": reference, "payload_status": payload_status})
    return Response(status=status.HTTP_400_BAD_REQUEST)



from django.conf import settings
import http.client
import json
from urllib.parse import urlparse
import pytz
import datetime

from rest_framework import status
from rest_framework.response import Response

ACCESS_TOKEN_URL = "/api/v2/access/token"
PAYMENT_CARDS_URL = "/api/v2/paymentcards"

SETTINGS_VARIABLES = [
    'TROPIPAY_CLIENT_ID',
    'TROPIPAY_CLIENT_SECRET',
    'TROPIPAY_CLIENT_EMAIL',
    'TROPIPAY_CLIENT_PASSWORD',
    'TROPIPAY_URL',
    'TROPIPAY_SUCCESS_URL',
    'TROPIPAY_FAILED_URL',
    'TROPIPAY_NOTIFICATION_URL',
    'TROPIPAY_FEE',
    'TROPIPAY_TIMEZONE',
    'PAYMENT_CURRENCY',
]

ORDER_KEYS = [
    "uuid",
    "title",
    "name",
    "last_name",
    "email",
    "phone_number",
    "description",
    "location",
    "total"
]

def create_message_for_order(order):
    mensaje = [
        'Purchase via Tropipay:',
        f'Ticket: {order.get("uuid")}',
        f'Buyer: {order.get("name")} {order.get("last_name")}',
        f'Buyer Phone Numer: {order.get("phone_number")}',
        f'Buyer Email: {order.get("email")}',
        f'Total: {order.get("total"):.2f}USD',
        f'Location Details: {order.get("location")}',
        f'Other Details: {order.get("description")}'
    ]
    return '\n'.join(mensaje)

def config_exists():
    return not any(not hasattr(settings, variable) for variable in SETTINGS_VARIABLES)


def get_config():
    return {variable: getattr(settings, variable) for variable in SETTINGS_VARIABLES}


def get_token(tropipay_config):
    conn = http.client.HTTPSConnection(urlparse(tropipay_config.get("TROPIPAY_URL")).netloc)
    payload_tpp = json.dumps({
        "grant_type": "client_credentials",
        "client_id": tropipay_config.get("TROPIPAY_CLIENT_ID"),
        "client_secret": tropipay_config.get("TROPIPAY_CLIENT_SECRET")
    })
    conn.request("POST", ACCESS_TOKEN_URL, payload_tpp, {'Content-Type': "application/json"})
    return json.loads(conn.getresponse().read().decode("utf-8")).get("access_token")

def get_payment_payload(tropipay_config, order):
    time = datetime.datetime.now(pytz.timezone(tropipay_config.get("TROPIPAY_TIMEZONE")))
    fee = order.get("total") * tropipay_config.get("TROPIPAY_FEE") / 100
    return {
        "reference": order.get("uuid"),
        "concept": order.get("title"),
        "favorite": "false",
        "description": create_message_for_order(order),
        "amount": round(round((order.get("total") + fee + 0.5), 2) * 100) if fee > 0 else round(
            round(order.get("total"), 2) * 100),
        "currency": tropipay_config.get("PAYMENT_CURRENCY"),
        "singleUse": "true",
        "reasonId": 4,
        "expirationDays": 0,
        "lang": "es",
        "urlSuccess": tropipay_config.get("TROPIPAY_SUCCESS_URL"),
        "urlFailed": tropipay_config.get("TROPIPAY_FAILED_URL"),
        "urlNotification": tropipay_config.get("TROPIPAY_NOTIFICATION_URL"),
        "serviceDate": f"{time.year} - {time.month} - {time.day}",
        "client": {
            'name': order.get("name"),
            'lastName': order.get("last_name"),
            "address": order.get("location"),
            "phone": order.get("phone_number"),
            "email": order.get("email"),
            "countryId": 0,
            "termsAndConditions": "true"
        },
        "directPayment": "true",
        "paymentMethods": ["EXT", "TPP"]
    }

def perform_payment(tropipay_config, payload):
    conn = http.client.HTTPSConnection(urlparse(tropipay_config.get("TROPIPAY_URL")).netloc)
    token = get_token(tropipay_config)
    conn.request("POST", PAYMENT_CARDS_URL, json.dumps(payload), {
        'Content-Type': "application/json",
        'Authorization': "Bearer " + token
    })
    response = json.loads(conn.getresponse().read().decode("utf-8"))
    if 'error' in response:
        return Response(status=status.HTTP_400_BAD_REQUEST)
    else:
        return Response(data={'payment_link': response.get('shortUrl', None)}, status=status.HTTP_200_OK)